]> git.lyx.org Git - lyx.git/blobdiff - src/Row.h
Merge branch 'master' of git.lyx.org:lyx
[lyx.git] / src / Row.h
index 2aa09a697e9e2e3f5231393fc3d829c4ba420c35..f69eeca9f17ae9b83e4d24fafbafdbb105d1a133 100644 (file)
--- a/src/Row.h
+++ b/src/Row.h
@@ -46,8 +46,6 @@ public:
                 * correspond to any paragraph contents
                 */
                VIRTUAL,
-               // A stretchable space, basically
-               SEPARATOR,
                // An inset
                INSET,
                // Some spacing described by its width, not a string
@@ -63,23 +61,40 @@ public:
                        : type(t), pos(p), endpos(p + 1), inset(0),
                          extra(0), font(f), change(ch), final(false) {}
 
-               // returns total width of element, including separator overhead
-               double width() const { return dim.wid + extra; };
-               // returns position in pixels (from the left) of position
-               // \param i in the row element.
-               double pos2x(pos_type const i) const;
+               // Return total width of element, including separator overhead
+               // FIXME: Cache this value or the number of separators?
+               double full_width() const { return dim.wid + extra * countSeparators(); }
+               // Return the number of separator in the element (only STRING type)
+               int countSeparators() const;
 
+               /** Return position in pixels (from the left) of position
+                * \param i in the row element.
+                */
+               double pos2x(pos_type const i) const;
                /** Return character position that is the closest to
                 *  pixel position \param x. The value \param x is
                 *  adjusted to the actual pixel position.
                */
-               pos_type x2pos(double &x) const;
+               pos_type x2pos(int &x) const;
+               /** Break the element if possible, so that its width is less
+                * than \param w. Returns true on success. When \param force
+                * is true, the string is cut at any place, other wise it
+                * respects the row breaking rules of characters.
+                */
+               bool breakAt(int w, bool force);
 
                // Returns the position on left side of the element.
                pos_type left_pos() const;
                // Returns the position on right side of the element.
                pos_type right_pos() const;
 
+               //
+               bool isRTL() const { return font.isVisibleRightToLeft(); }
+               // This is true for virtual elements.
+               // Note that we do not use the type here. The two definitions
+               // should be equivalent
+               bool isVirtual() const { return pos == endpos; }
+
                // The kind of row element
                Type type;
                // position of the element in the paragraph
@@ -93,10 +108,10 @@ public:
                // Non-zero only if element is an inset
                Inset const * inset;
 
-               // Only non-null for separator elements
+               // Only non-null for justified rows
                double extra;
 
// Non-empty if element is a string or separator
                docstring str;
                //
                Font font;
@@ -130,6 +145,10 @@ public:
        void setSelectionAndMargins(DocIterator const & beg,
                DocIterator const & end) const;
 
+       ///
+       void pit(pit_type p) { pit_ = p; }
+       ///
+       pit_type pit() const { return pit_; }
        ///
        void pos(pos_type p) { pos_ = p; }
        ///
@@ -149,13 +168,23 @@ public:
        Dimension & dimension() { return dim_; }
        ///
        int height() const { return dim_.height(); }
-       ///
+       /// The width of the row, including the left margin, but not the right one.
        int width() const { return dim_.wid; }
        ///
        int ascent() const { return dim_.asc; }
        ///
        int descent() const { return dim_.des; }
 
+       /// The offset of the left-most cursor position on the row
+       int left_x() const;
+       /// The offset of the right-most cursor position on the row
+       int right_x() const;
+
+       // Return the number of separators in the row
+       int countSeparators() const;
+       // Set the extra spacing for every separator in STRING elements
+       void setSeparatorExtraWidth(double w);
+
        ///
        void add(pos_type pos, Inset const * ins, Dimension const & dim,
                 Font const & f, Change const & ch);
@@ -166,9 +195,6 @@ public:
        void addVirtual(pos_type pos, docstring const & s,
                        Font const & f, Change const & ch);
        ///
-       void addSeparator(pos_type pos, char_type const c,
-                         Font const & f, Change const & ch);
-       ///
        void addSpace(pos_type pos, int width, Font const & f, Change const & ch);
 
        ///
@@ -206,8 +232,9 @@ public:
         * remains is a large word, cut it to \param width.
         * \param body_pos minimum amount of text to keep.
         * \param width maximum width of the row
+        * \return true if the row has been shortened.
         */
-       void shorten_if_needed(pos_type const body_pos, int const width);
+       bool shortenIfNeeded(pos_type const body_pos, int const width);
 
        /**
         * If last element of the row is a string, compute its width
@@ -223,12 +250,12 @@ public:
 
        friend std::ostream & operator<<(std::ostream & os, Row const & row);
 
-       /// width of a separator (i.e. space)
+       /// additional width for separators in justified rows (i.e. space)
        double separator;
        /// width of hfills in the label
        double label_hfill;
-       /// the x position of the row (left margin)
-       double x;
+       /// the left margin position of the row
+       int left_margin;
        /// the right margin of the row
        int right_margin;
        ///
@@ -264,6 +291,8 @@ private:
        mutable bool changed_;
        /// CRC of row contents.
        mutable size_type crc_;
+       /// Index of the paragraph that contains this row
+       pit_type pit_;
        /// first pos covered by this row
        pos_type pos_;
        /// one behind last pos covered by this row
@@ -93,10 +108,10 @@ public:
                double extra;
 
-               // Non-empty if element is a string or separator
+               // Non-empty if element is a string or is virtual
                docstring str;
                //